import responses
from pathlib import Path

import aiod
from aiod.configuration import config


resources_path = Path(__file__).parent / "resources"


def test_counts():
    with responses.RequestsMock() as mocked_requests:
        with open(resources_path / "counts.json", "r") as f:
            res_body = f.read()
        mocked_requests.add(
            responses.GET,
            url=f"{config.api_base_url}counts/{config.version}",
            body=res_body,
            status=200,
        )
        counts = aiod.counts(data_format="json")

        assert isinstance(counts, dict)
        assert "datasets" in counts



import asyncio
import pytest
import responses

from aioresponses import aioresponses
from pathlib import Path
from typing import Callable

import aiod

from aiod.configuration import config

resources_path = Path(__file__).parent / "resources"


asset_names = [
    "case_studies",
    "computational_assets",
    "contacts",
    "datasets",
    "educational_resources",
    "events",
    "experiments",
    "ml_models",
    "news",
    "organisations",
    "persons",
    "platforms",
    "projects",
    "publications",
    "services",
    "teams",
]

assets_with_search = [
    "datasets",
    "events",
    "experiments",
    "ml_models",
    "news",
    "organisations",
    "projects",
    "publications",
    "services",
]


@pytest.fixture(params=asset_names)
def asset_name(request):
    return request.param


@pytest.fixture(params=assets_with_search)
def asset_with_search(request):
    return request.param


def test_common_endpoints_are_created(asset_name: str):
    asset = getattr(aiod, asset_name)
    assert isinstance(getattr(asset, "get_list"), Callable)
    assert isinstance(getattr(asset, "counts"), Callable)
    assert isinstance(getattr(asset, "get_asset"), Callable)
    assert isinstance(getattr(asset, "get_asset_from_platform"), Callable)
    assert isinstance(getattr(asset, "get_content"), Callable)
    assert isinstance(getattr(asset, "get_list_async"), Callable)
    assert isinstance(getattr(asset, "get_assets_async"), Callable)


def test_search_endpoints_are_created(asset_with_search: str):
    asset = getattr(aiod, asset_with_search)
    assert isinstance(getattr(asset, "search"), Callable)


def test_endpoint_get_list(asset_name):
    with responses.RequestsMock() as mocked_requests:
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}{asset_name}/{config.version}?offset=0&limit=10",
            body=b'[{"resource_1": "info"},{"resource_2": "info"}]',
            status=200,
        )
        endpoint = getattr(aiod, asset_name)
        metadata_list = endpoint.get_list()

        assert len(metadata_list) == 2


def test_endpoint_counts(asset_name):
    with responses.RequestsMock() as mocked_requests:
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}counts/{asset_name}/{config.version}?detailed=false",
            body=b"2",
            status=200,
        )
        endpoint = getattr(aiod, asset_name)
        counts = endpoint.counts()

        assert counts == 2


def test_endpoint_get_asset(asset_name):
    with responses.RequestsMock() as mocked_requests:
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}{asset_name}/{config.version}/1",
            body=b'{"resource":"fake_details"}',
            status=200,
        )
        endpoint = getattr(aiod, asset_name)
        metadata = endpoint.get_asset(identifier=1, data_format="json")

        assert metadata == {"resource": "fake_details"}


def test_endpoint_get_list_from_platform(asset_name):
    platform_name = "zenodo"
    with responses.RequestsMock() as mocked_requests:
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}platforms/{platform_name}/{asset_name}/{config.version}?offset=0&limit=10",
            body=b'[{"resource_1": "info"},{"resource_2": "info"}]',
            status=200,
        )
        endpoint = getattr(aiod, asset_name)
        metadata_list = endpoint.get_list(platform=platform_name)

        assert len(metadata_list) == 2


def test_endpoint_get_asset_from_platform(asset_name):
    platform_name = "zenodo"
    platform_identifier = "zenodo.org:1000"
    with responses.RequestsMock() as mocked_requests:
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}platforms/{platform_name}/{asset_name}/{config.version}/{platform_identifier}",
            body=b'{"resource":"fake_details"}',
            status=200,
        )
        endpoint = getattr(aiod, asset_name)
        metadata = endpoint.get_asset_from_platform(
            platform=platform_name,
            platform_identifier=platform_identifier,
            data_format="json",
        )

        assert metadata == {"resource": "fake_details"}


def test_get_content(asset_name):
    with responses.RequestsMock() as mocked_requests:
        with open(resources_path / "content.csv", "r") as f:
            res_body = f.read()
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}{asset_name}/{config.version}/1/content",
            body=res_body,
            status=200,
        )
        endpoint = getattr(aiod, asset_name)
        content = endpoint.get_content(identifier=1)

        assert content == b"a,b,c\n1,2,3"


def test_get_content_with_distribution_idx(asset_name):
    with responses.RequestsMock() as mocked_requests:
        with open(resources_path / "content.csv", "r") as f:
            res_body = f.read()
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}{asset_name}/{config.version}/1/content/2",
            body=res_body,
            status=200,
        )
        endpoint = getattr(aiod, asset_name)
        content = endpoint.get_content(identifier=1, distribution_idx=2)

        assert content == b"a,b,c\n1,2,3"


def test_search(asset_with_search):
    search_query = "my query"
    search_field = "name"
    platforms = ["aiod", "openml"]
    get_all = True

    query = (
        f"?search_query={search_query.replace(' ', '+')}&platforms={platforms[0]}"
        f"&platforms={platforms[1]}&offset=0&limit=10&search_fields={search_field}&get_all=true"
    )
    with responses.RequestsMock() as mocked_requests:
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}search/{asset_with_search}/{config.version}{query}",
            body=b'{"total_hits": 2,"resources": [{"resource_1": "info"},{"resource_2": "info"}],"limit": 10,"offset": 0}',
            status=200,
        )
        endpoint = getattr(aiod, asset_with_search)
        metadata_list = endpoint.search(
            query=search_query,
            search_field=search_field,
            platforms=platforms,
            get_all=get_all,
            data_format="json",
        )

        assert len(metadata_list) == 2
        assert metadata_list == [{"resource_1": "info"}, {"resource_2": "info"}]


def test_endpoint_get_asset_async(asset_name):
    loop = asyncio.get_event_loop()
    with aioresponses() as mocked_responses:
        mocked_responses.get(
            f"{config.api_base_url}{asset_name}/{config.version}/1",
            payload={"resource": "fake_details"},
            status=200,
        )
        mocked_responses.get(
            config.api_base_url + f"{asset_name}/" + config.version + "/" + "3",
            payload={"resource": "fake_details_2"},
            status=200,
        )

        endpoint = getattr(aiod, asset_name)
        metadata = loop.run_until_complete(
            endpoint.get_assets_async(identifiers=[1, 3], data_format="json")
        )
        assert metadata == [
            {"resource": "fake_details"},
            {"resource": "fake_details_2"},
        ]


def test_endpoint_get_list_async(asset_name):
    loop = asyncio.get_event_loop()
    with aioresponses() as mocked_responses:
        mocked_responses.get(
            f"{config.api_base_url}{asset_name}/{config.version}?offset=0&limit=2",
            payload=[{"resource_1": "info"}, {"resource_2": "info"}],
            status=200,
        )
        mocked_responses.get(
            f"{config.api_base_url}{asset_name}/{config.version}?offset=2&limit=1",
            payload=[{"resource_3": "info"}],
            status=200,
        )

        endpoint = getattr(aiod, asset_name)
        metadata = loop.run_until_complete(
            endpoint.get_list_async(offset=0, limit=3, batch_size=2, data_format="json")
        )
        assert metadata == [
            {"resource_1": "info"},
            {"resource_2": "info"},
            {"resource_3": "info"},
        ]



import aiohttp
import asyncio
import requests

import pandas as pd
from typing import Literal
from functools import partial

from aiod.calls.urls import (
    url_to_get_asset,
    url_to_get_content,
    url_to_get_list,
    url_to_get_list_from_platform,
    url_to_get_asset_from_platform,
    url_to_resource_counts,
    url_to_search,
)

from aiod.calls.utils import format_response, wrap_calls


def get_list(
    *,
    asset_type: str,
    platform: str | None = None,
    offset: int = 0,
    limit: int = 10,
    version: str | None = None,
    data_format: Literal["pandas", "json"] = "pandas",
) -> pd.DataFrame | list[dict]:
    """
    Retrieve a list of ASSET_TYPE from the catalogue.

    Parameters:
        platform: Return metadata of ASSET_TYPE assets of this platform (default is None).
        offset: The offset for pagination (default is 0).
        limit: The maximum number of items to retrieve (default is 10).
        version: The version of the endpoint (default is None).
        data_format: The desired format for the response (default is "pandas").
            For "json" formats, the returned type is a json decoded type, i.e. in this case a list of dicts.

    Returns:
        The retrieved metadata in the specified format.
    """
    url = (
        url_to_get_list_from_platform(asset_type, platform, offset, limit, version)
        if platform is not None
        else url_to_get_list(asset_type, offset, limit, version)
    )
    res = requests.get(url)
    resources = format_response(res.json(), data_format)
    return resources


def counts(
    *, asset_type: str, version: str | None = None, per_platform: bool = False
) -> int | dict[str, int]:
    """
    Retrieve the number of ASSET_TYPE assets in the metadata catalogue.

    Parameters:
        version: The version of the endpoint (default is None).
        per_platform: Whether to list counts per platform (default is False).

    Returns:
        The number ASSET_TYPE assets in the metadata catalogue. If the parameter per_platform is True, it returns a dictionary with platform names as keys and the number of ASSET_TYPE assets from that platform as values.
    """

    url = url_to_resource_counts(version, per_platform, asset_type)
    res = requests.get(url)
    return res.json()


def get_asset(
    identifier: int,
    *,
    asset_type: str,
    version: str | None = None,
    data_format: Literal["pandas", "json"] = "pandas",
) -> pd.Series | dict:
    """
    Retrieve metadata for a specific ASSET_TYPE.

    Parameters:
        identifier: The identifier of the ASSET_TYPE to retrieve.
        version: The version of the endpoint (default is None).
        data_format: The desired format for the response (default is "pandas").
            For "json" formats, the returned type is a json decoded type, in this case a dict.

    Returns:
        The retrieved metadata for the specified ASSET_TYPE.
    """
    url = url_to_get_asset(asset_type, identifier, version)
    res = requests.get(url)
    resources = format_response(res.json(), data_format)
    return resources


def get_asset_from_platform(
    *,
    platform: str,
    platform_identifier: str,
    asset_type: str,
    version: str | None = None,
    data_format: Literal["pandas", "json"] = "pandas",
) -> pd.Series | dict:
    """
    Retrieve metadata for a specific ASSET_TYPE identified by the external platform identifier.

    Parameters:
        platform: The platform where the ASSET_TYPE asset is retrieved from.
        platform_identifier: The identifier under which the ASSET_TYPE is known by the platform.
        version: The version of the endpoint (default is None).
        data_format: The desired format for the response (default is "pandas").
            For "json" formats, the returned type is a json decoded type, in this case a dict.

    Returns:
        The retrieved metadata for the specified ASSET_TYPE.
    """
    url = url_to_get_asset_from_platform(
        asset_type, platform, platform_identifier, version
    )
    res = requests.get(url)
    resources = format_response(res.json(), data_format)
    return resources


def get_content(
    *,
    identifier: int,
    asset_type: str,
    distribution_idx: int = 0,
    version: str | None = None,
) -> bytes:
    """
    Retrieve the data content of a specific ASSET_TYPE.

    Parameters:
        identifier: The identifier of the ASSET_TYPE asset.
        distribution_idx: The index of a specific distribution from the distribution list (default is 0).
        version: The version of the endpoint (default is None).

    Returns:
        The data content for the specified ASSET_TYPE.
    """
    url = url_to_get_content(asset_type, identifier, distribution_idx, version)
    res = requests.get(url)
    distribution = res.content
    return distribution


def search(
    query: str,
    *,
    platforms: list[str] | None = None,
    offset: int = 0,
    limit: int = 10,
    search_field: (
        None | Literal["name", "issn", "description_html", "description_plain"]
    ) = None,
    get_all: bool = True,
    version: str | None = None,
    data_format: Literal["pandas", "json"] = "pandas",
    asset_type: str,
) -> pd.DataFrame | list[dict]:
    """
    Search metadata for ASSET_TYPE type using the Elasticsearch endpoint of the AIoD metadata catalogue.

    Parameters:
        search: The string to be matched against the search fields.
        platforms: The platforms to filter the search results.
            If None, results from all platforms will be returned (default is None).
        offset: The offset for pagination (default is 0).
        limit: The maximum number of results to retrieve (default is 10).
        search_field:
            The specific fields to search within. If None, the query will be matched against all fields (default is None).
        get_all: If true, a request to the database is made to retrieve all data.
            If false, only the indexed information is returned. (default is True).
        version: The version of the endpoint to use (default is None).
        data_format: The desired format for the response (default is "pandas").
            For "json" formats, the returned type is a json decoded type, in this case a list of dict's.

    Returns:
        The retrieved metadata in the specified format.
    """
    url = url_to_search(
        asset_type,
        query,
        platforms,
        offset,
        limit,
        search_field,
        get_all,
        version,
    )
    res = requests.get(url)
    resources = format_response(res.json()["resources"], data_format)
    return resources


async def get_assets_async(
    identifiers: list[int],
    *,
    asset_type: str,
    version: str | None = None,
    data_format: Literal["pandas", "json"] = "pandas",
) -> pd.DataFrame | list[dict]:
    """
    Asynchronously retrieve metadata for a list of ASSET_TYPE identifiers.

    Parameters:
        identifiers: The list of identifiers of the ASSET_TYPE to retrieve.
        version: The version of the endpoint (default is None).
        data_format: The desired format for the response (default is "pandas").
            For "json" formats, the returned type is a json decoded type, in this case a list of dicts.

    Returns:
        The retrieved metadata for the specified ASSET_TYPE.
    """
    urls = [
        url_to_get_asset(asset_type, identifier, version) for identifier in identifiers
    ]
    response_data = await _fetch_resources(urls)
    resources = format_response(response_data, data_format)
    return resources


async def get_list_async(
    *,
    asset_type: str,
    offset: int = 0,
    limit: int = 100,
    batch_size: int = 10,
    version: str | None = None,
    data_format: Literal["pandas", "json"] = "pandas",
) -> pd.DataFrame | list[dict]:
    """
    Asynchronously retrieve a list of ASSET_TYPE from the catalogue in batches.

    Parameters:
        offset: The offset for pagination (default is 0).
        limit: The maximum number of items to retrieve (default is 10).
        batch_size: The number of items in a a batch.
        version: The version of the endpoint (default is None).
        data_format: The desired format for the response (default is "pandas").
            For "json" formats, the returned type is a json decoded type, in this case a list of dicts.

    Returns:
        The retrieved metadata in the specified format.

    Raises:
        ValueError: Batch size must be larger than 0.
    """
    if batch_size <= 0:
        raise ValueError(
            "batch_size must be larger than 0, otherwise you can use the synchronous get_list function!"
        )

    offsets = range(offset, offset + limit, batch_size)
    last_batch_size = (limit % batch_size) or batch_size
    batch_sizes = [batch_size] * (len(offsets) - 1) + [last_batch_size]

    urls = [
        url_to_get_list(asset_type, offset, limit, version)
        for offset, limit in zip(offsets, batch_sizes)
    ]

    response_data = await _fetch_resources(urls)
    flattened_response_data = [
        response for batch in response_data for response in batch
    ]
    resources = format_response(flattened_response_data, data_format)
    return resources


async def _fetch_resources(urls) -> dict:
    async def _fetch_data(session, url) -> dict:
        async with session.get(url) as response:
            return await response.json()

    async with aiohttp.ClientSession() as session:
        tasks = [_fetch_data(session, url) for url in urls]
        response_data = await asyncio.gather(*tasks)
    return response_data


wrap_common_calls = partial(
    wrap_calls,
    calls=[
        get_list,
        counts,
        get_asset,
        get_asset_from_platform,
        get_content,
        get_assets_async,
        get_list_async,
    ],
)
wrap_search_call = partial(wrap_calls, calls=[search])



import urllib

from typing import Literal

from aiod.configuration import config


def url_to_get_asset(
    asset_type: str, identifier: int, version: str | None = None
) -> str:
    version = version or config.version
    url = f"{config.api_base_url}{asset_type}/{version}/{identifier}"
    return url


def url_to_get_list(
    asset_type: str, offset: int = 0, limit: int = 10, version: str | None = None
) -> str:

    query = urllib.parse.urlencode({"offset": offset, "limit": limit})
    version = version or config.version
    url = f"{config.api_base_url}{asset_type}/{version}?{query}"
    return url


def url_to_search(
    asset_type: str,
    search_query: str,
    platforms: list[str] | None = None,
    offset: int = 0,
    limit: int = 10,
    search_fields: (
        None | Literal["name", "issn", "description_html", "description_plain"]
    ) = None,
    get_all: bool = True,
    version: str | None = None,
) -> str:
    query_params = {
        key: value
        for key, value in locals().items()
        if value is not None and value != "" and key not in ["version", "asset_type"]
    }
    query = urllib.parse.urlencode(query_params, doseq=True).lower()
    version = version or config.version
    url = f"{config.api_base_url}search/{asset_type}/{version}?{query}"
    return url


def url_to_resource_counts(
    version: str | None = None,
    per_platform: bool = False,
    asset_type: str | None = None,
) -> str:
    query = urllib.parse.urlencode({"detailed": per_platform}).lower()
    version = version or config.version
    url = f"{config.api_base_url}counts/{asset_type}/{version}?{query}"
    return url


def url_to_get_content(
    asset_type: str,
    identifier: int,
    distribution_idx: int,
    version: str | None = None,
) -> str:
    version = version or config.version
    url = f"{config.api_base_url}{asset_type}/{version}/{identifier}/content"
    url += f"/{distribution_idx}" if distribution_idx else ""
    return url


def url_to_get_list_from_platform(
    asset_type: str,
    platform: str,
    offset: int = 0,
    limit: int = 10,
    version: str | None = None,
) -> str:

    query = urllib.parse.urlencode({"offset": offset, "limit": limit})
    version = version or config.version

    url = f"{config.api_base_url}platforms/{platform}/{asset_type}/{version}?{query}"
    return url


def url_to_get_asset_from_platform(
    asset_type: str,
    platform: str,
    platform_identifier: str,
    version: str | None = None,
) -> str:
    version = version or config.version
    url = f"{config.api_base_url}platforms/{platform}/{asset_type}/{version}/{platform_identifier}"
    return url



import pandas as pd

from functools import partial, update_wrapper
from typing import Callable, Literal, Tuple


def format_response(
    response: list | dict, data_format: Literal["pandas", "json"]
) -> pd.Series | pd.DataFrame | dict | list:
    """
    Format the response data based on the specified format.

    Parameters:
        response (list | dict): The response data to format.
        data_format (Literal["pandas", "json"]): The desired format for the response.
            For "json" formats, the returned type is a json decoded type, i.e. a dict or a list.

    Returns:
        pd.Series | pd.DataFrame | dict: The formatted response data.

    Raises:
        Exception: If the specified format is invalid or not supported.
    """

    if data_format == "pandas":
        if isinstance(response, dict):
            return pd.Series(response)
        if isinstance(response, list):
            return pd.DataFrame(response)
    elif data_format == "json" and (
        isinstance(response, dict) or isinstance(response, list)
    ):
        return response

    raise Exception(
        f"Format: {data_format} invalid or not supported for responses of {type(response)=}."
    )


def wrap_calls(asset_type: str, calls: list[Callable], module: str) -> Tuple[Callable, ...]:
    wrapper_list = []
    for wrapped in calls:
        wrapper: Callable = partial(wrapped, asset_type=asset_type)
        wrapper = update_wrapper(wrapper, wrapped)
        wrapper.__doc__ = (
            wrapped.__doc__.replace("ASSET_TYPE", asset_type)
            if wrapped.__doc__ is not None
            else ""
        )
        wrapper.__module__ = module
        wrapper_list.append(wrapper)

    return tuple(wrapper_list)



from collections import defaultdict
from dataclasses import dataclass, field
from typing import Any, Callable, TypeAlias


AttributeObserver: TypeAlias = Callable[[str, Any, Any], None]


@dataclass
class Config:
    api_base_url: str
    version: str
    auth_server_url: str
    realm: str
    client_id: str
    access_token: str | None = None
    refresh_token: str | None = None

    _observers: dict[str, set[AttributeObserver]] = field(default_factory=lambda: defaultdict(set))

    def subscribe(self, attribute: str, on_change: AttributeObserver):
        if on_change not in self._observers[attribute]:
            self._observers[attribute].add(on_change)

    def __setattr__(self, key, value):
        old_value = getattr(self, key, None)
        super().__setattr__(key, value)

        if hasattr(self, "_observers"):
            for observer in self._observers[key]:
                observer(key, old_value, value)


config = Config(
    api_base_url="https://api.aiod.eu/",
    version="v1",
    auth_server_url="https://auth.aiod.eu/aiod-auth/",
    realm="aiod",
    client_id="aiod-sdk",
)



from aiod.authentication import login, logout, get_current_user
from aiod.configuration import config
from aiod.resources import case_studies
from aiod.resources import computational_assets
from aiod.resources import contacts
from aiod.default.counts import asset_counts as counts
from aiod.resources import datasets
from aiod.resources import educational_resources
from aiod.resources import events
from aiod.resources import experiments
from aiod.resources import ml_models
from aiod.resources import news
from aiod.resources import organisations
from aiod.resources import persons
from aiod.resources import platforms
from aiod.resources import projects
from aiod.resources import publications
from aiod.resources import services
from aiod.resources import teams



import http.client
import requests
from keycloak import KeycloakOpenID, KeycloakAuthenticationError, KeycloakPostError
from typing import Sequence, NamedTuple

from aiod.configuration import config


def _connect_keycloak() -> KeycloakOpenID:
    return KeycloakOpenID(
        server_url=config.auth_server_url,
        client_id=config.client_id,
        realm_name=config.realm,
    )


def keycloak_openid() -> KeycloakOpenID:
    global _keycloak_openid
    if _keycloak_openid is None:
        _keycloak_openid = _connect_keycloak()
    return _keycloak_openid


def on_keycloak_config_changed(_: str, __: str, ___: str) -> None:
    global _keycloak_openid
    logout(ignore_post_error=True)
    _keycloak_openid = None


config.subscribe("auth_server_url", on_change=on_keycloak_config_changed)
config.subscribe("realm", on_change=on_keycloak_config_changed)
config.subscribe("client_id", on_change=on_keycloak_config_changed)

_keycloak_openid: KeycloakOpenID | None = None


class User(NamedTuple):
    name: str
    roles: Sequence[str]


def login(username: str, password: str) -> None:
    """Logs in the user with the provided username and password.

    Args:
        username (str): The username of the user.
        password (str): The password of the user.

    Raises:
        ValueError: When username or password is empty.
        FailedAuthenticationError: When the username or password is wrong.
    """
    if not username:
        raise ValueError(
            f"{username!r} is not a valid username, must be a non-empty string."
        )
    if not password:
        raise ValueError(
            f"{password!r} is not a valid password, must be a non-empty string."
        )

    try:
        token = keycloak_openid().token(username, password)
    except KeycloakAuthenticationError as e:
        raise FailedAuthenticationError(
            "Authentication failed! Please verify your credentials."
        ) from e
    config.access_token = token["access_token"]
    config.refresh_token = token["refresh_token"]


def logout(ignore_post_error: bool = False) -> None:
    """Logs out the current user.

    Args:
        ignore_post_error:
            If true, do not raise an error if the logout attempt failed.

    """
    try:
        keycloak_openid().logout(config.refresh_token)
    except KeycloakPostError as e:
        if not ignore_post_error:
            raise e

    config.access_token = None
    config.refresh_token = None


def get_current_user() -> User:
    """Return name and roles of the user that is currently authenticated.

    Returns:
        User: The user information for the currently authenticated user.

    Raises:
        NotAuthenticatedError: When the user is not authenticated.
    """
    response = requests.get(
        f"{config.api_base_url}authorization_test",
        headers={"Authorization": f"Bearer {config.access_token}"},
    )

    content = response.json()
    if response.status_code == http.client.UNAUTHORIZED:
        raise NotAuthenticatedError(content)
    return User(
        name=content["name"],
        roles=tuple(content["roles"]),
    )


class FailedAuthenticationError(Exception):
    """Raised when an authentication error occurred."""


class NotAuthenticatedError(Exception):
    """Raised when an endpoint that requires authentication is called without authentication."""



import pytest
import responses

from unittest.mock import Mock

import aiod
from aiod.configuration import config
from aiod.authentication.authentication import keycloak_openid


@pytest.fixture
def mocked_token() -> Mock:
    token = {"access_token": "fake_token", "refresh_token": "fake_refresh_token"}
    return Mock(return_value=token)


@pytest.fixture
def mocked_logout() -> Mock:
    return Mock()


def test_authentication(mocked_token: Mock, mocked_logout: Mock):
    keycloak_openid().token = mocked_token
    keycloak_openid().logout = mocked_logout

    access_token = config.access_token
    refresh_token = config.refresh_token
    assert access_token is None, access_token
    assert refresh_token is None, refresh_token

    aiod.login("fake_username", "fake_p455w0rd")
    access_token = config.access_token
    refresh_token = config.refresh_token
    assert access_token == "fake_token", access_token
    assert refresh_token == "fake_refresh_token", refresh_token

    aiod.logout()
    access_token = config.access_token
    refresh_token = config.refresh_token
    assert access_token is None, access_token
    assert refresh_token is None, refresh_token


def test_get_user_endpoint(mocked_token: Mock):
    keycloak_openid().token = mocked_token

    with responses.RequestsMock() as mocked_requests:
        mocked_requests.add(
            responses.GET,
            f"{config.api_base_url}authorization_test",
            json={"name": "user", "roles": ["a_role"]},
            status=200,
        )
        aiod.login("fake_username", "fakeP455w0rd")
        user = aiod.get_current_user()

        header = mocked_requests.calls[0].request.headers["Authorization"]
        assert header == "Bearer fake_token", header
        assert user.name == "user", user
        assert user.roles == ("a_role",), user



import pandas as pd
import requests
from typing import Literal

from aiod.configuration import config


def asset_counts(
    version: str | None = None, data_format: Literal["pandas", "json"] = "pandas"
) -> pd.DataFrame | dict:
    """
    Retrieve counts of assets.

    This method sends a GET request to the counts endpoint to retrieve counts of assets.

    Parameters:
        version (str | None): The version of the counts endpoint (default is None).
        format (Literal["pandas", "json"]): The desired format for the response (default is "pandas").
            For "json" format, the returned type is a json decoded type, in this case a dict.

    Returns:
        pd.DataFrame | dict: Counts as a Pandas data frame or a dictionary.
    """
    version = version or config.version
    url = f"{config.api_base_url}counts"
    if version:
        url += "/" + version

    res = requests.get(url)

    if data_format == "pandas":
        return pd.DataFrame(res.json())
    elif data_format == "json" and isinstance(res.json(), dict):
        return res.json()

    raise Exception(
        f"Format: {data_format} invalid or not supported for responses of {type(res.json())=}."
    )



from aiod.calls import calls

(
    get_list,
    counts,
    get_asset,
    get_asset_from_platform,
    get_content,
    get_assets_async,
    get_list_async,
) = calls.wrap_common_calls(asset_type="events", module=__name__)

(search,) = calls.wrap_search_call(asset_type="events", module=__name__)
